from typing import Dict, Tuple
from xlsxwriter.format import Format
from xlsxwriter.workbook import Workbook
from xlsxwriter.worksheet import Worksheet

from .semestre import Semestre
from .funciones_curso import escribir_curso


def escribir_semestre_listado(semestre: Semestre, cuaderno: Workbook, hoja: Worksheet, fila: int, columna: int,
                              formatos: Dict[str, Format]) -> Tuple[int, int]:
    # print('Escribir semestre', semestre.get_numero())
    profundidad = semestre.obtener_maximo_historial()
    lista_siglas = semestre.get_siglas()
    # print('Max. historial : ' + str(profundidad))

    for id_sigla, sigla in enumerate(lista_siglas):
        # print(id_sigla, sigla)
        escribir_curso(semestre.get_cursos_por_siglas()[sigla], cuaderno, hoja, fila, id_sigla * 9, formatos,
                       profundidad)

    # print(profundidad + fila, columna)
    return profundidad + fila + 5, columna


def escribir_semestre(semestre: Semestre, cuaderno: Workbook, hoja: Worksheet, fila: int, columna: int,
                      formatos: Dict[str, Format]) -> Tuple[int, int]:
    # print('Escribir semestre', semestre.get_numero())
    profundidad = semestre.obtener_maximo_historial()
    lista_siglas = semestre.get_siglas()
    # print('Max. historial : ' + str(profundidad))

    for id_sigla, sigla in enumerate(lista_siglas):
        # print(id_sigla, sigla)
        escribir_curso(semestre.get_cursos_por_siglas()[sigla], cuaderno, hoja, fila, id_sigla * 9, formatos,
                       profundidad)

    # print(profundidad + fila, columna)
    return profundidad + fila + 5, columna



import os
import pyperclip
import xlsxwriter
import getpass
import urllib3
import ssl

from datetime import datetime
from typing import Dict
from termcolor import cprint
from xlsxwriter.format import Format
from xlsxwriter.workbook import Workbook
from xlsxwriter.worksheet import Worksheet

from .config import *
from .funciones_procesamiento_expediente import cargar_cursos_carrera
from .funciones_memory_reader import identificar_tipo, procesar_cursos_solicitados, procesar_expediente
from .funciones_io import leer_historial
from .funciones_xlsxwriter import generar_formatos
from .funciones_expediente import escribir_encabezado_expediente, escribir_expediente
from .funciones_web_scraping import iniciar_proceso_descarga
from .funciones_procesamiento_expediente import procesar_archivos_expedientes
from .funciones_consola import clear, leer_rango_numeros_enteros
from .custom_http_adapter import CustomHttpAdapter, get_legacy_session


def imprimir_finalizado() -> None:
    print("FINALIZADO")
    print("SI LA APLICACIÓN FUE DE UTILIDAD PUEDE DONAR")
    print("EL EQUIVALENTE A UN CAFECITO AL SINPE MOVIL 50123456 (SI ES UN NUMERO DE VERDAD)")
    print("SI REALMENTE LE GUSTO CONSIDERE DONAR UN WHOPPER")
    print("SI TIENE DUDAS, NO LLAME, NO WHATSAPP SOLO RESPONDO TELEGRAM")
    print("SIGA EL CANAL DE YT https://www.youtube.com/mauricioz7")


def imprimir_sin_datos_expediente() -> None:
    print("EXPEDIENTE SIN DATOS")


def imprimir_no_hay_datos() -> None:
    print("NO HAY DATOS PARA PROCESAR")


def imprimir_procesando_expediente() -> None:
    print("EXPEDIENTE PROCESADO")


def imprimir_procesando_cursos_solicitados() -> None:
    print("SOLICITUD PROCESADO")


def imprimir_generador_prematriculas() -> None:
    # https://patorjk.com/software/taag/#p=display&f=Graffiti&t=Type%20Something%20
    print("PROCESADOR DE PREMATRICULAS")
    print("Desarrollado por Mauricio Andrés Zamora Hernández")
    print("Versión 2022 ALFA")


def opcion_menu_procesar_memoria() -> None:
    imprimir_generador_prematriculas()
    sigla_cursos, semestre_cursos, expediente = cargar_cursos_carrera(detalle_cursos)

    texto: str = pyperclip.paste()
    procesar: bool = False

    tipo: str = identificar_tipo(texto)
    if tipo == 'pre':
        imprimir_procesando_cursos_solicitados()
        carne, nombre = procesar_cursos_solicitados(texto)
        imprimir_finalizado()
        procesar = True
    elif tipo == 'exp':
        imprimir_procesando_expediente()
        carne, nombre = procesar_expediente(texto)
        imprimir_finalizado()
        procesar = True
    else:
        imprimir_no_hay_datos()

    if procesar:
        procesar_archivos_expedientes()

    input('Presione enter para continuar')


def opcion_menu_pruebas() -> None:
    print('modo excel')
    carne = 'C03370'
    print('carne', carne)

    sigla_cursos, semestre_cursos, expediente = cargar_cursos_carrera(detalle_cursos)
    expediente.set_carne(carne)

    lineas = leer_historial(carne)
    for linea in lineas:
        expediente.agregar_curso(linea)

    existe_directorio: bool = os.path.isdir('./salida')
    if not existe_directorio:
        os.mkdir('./salida')
    workbook: Workbook = xlsxwriter.Workbook(
        os.path.join('./salida', '{}.xlsx'.format(carne)))

    worksheet: Worksheet = workbook.add_worksheet('malla')
    worksheet.hide_gridlines(2)

    formatos: Dict[str, Format] = generar_formatos(workbook)
    r, c = escribir_encabezado_expediente(expediente, workbook, worksheet, 0, 0, formatos)
    expediente.procesar_requisitos_correquisitos()
    escribir_expediente(expediente, workbook, worksheet, r, c, formatos)
    workbook.close()

    input('Presione enter para continuar')


def opcion_menu_informacion() -> None:
    ancho = 60
    print('{:{align}{width}}'.format('ESTA ES UNA VERSIÓN DE PRUEBA', align='^', width=ancho))
    print('{:{align}{width}}'.format('EN CADA PREMATRICULA ALGO SE AGREGA O CORRIJE', align='^', width=ancho))
    print()
    print('{:{align}{width}}'.format('ESTE SOFTWARE ES DE TIPO "AS IS"', align='^', width=ancho))
    print('{:{align}{width}}'.format('https://en.wikipedia.org/wiki/As_is', align='^', width=ancho))
    print()
    print('{:{align}{width}}'.format('Este script fue hecho en mi tiempo libre,', align='^', width=ancho))
    print('{:{align}{width}}'.format('si quieren invitar a un combo de BK por', align='^', width=ancho))
    print('{:{align}{width}}'.format('semestre que lo usen se les agradece.', align='^', width=ancho))
    print()
    print('{:{align}{width}}'.format('por Mauricio Zamora', align='^', width=ancho))
    print('{:{align}{width}}'.format('mauricio@zamora.cr', align='^', width=ancho))
    input('Presione enter para continuar')


def opcion_menu_descargar() -> None:
    print('POR FAVOR ESCRIBA SUS CREDENCIALES DE LA UCR')
    print('Usuario: (como el correo, pero sin el @ucr.ac.cr')
    data['user'] = input('')
    print('Usuario: ', data['user'])
    try:
        data['password'] = getpass.getpass()
    except Exception as error:
        print('ERROR', error)

    if iniciar_proceso_descarga():
        procesar_archivos_expedientes()
    input('Presione enter para continuar')


def opcion_menu_salir() -> None:
    print(':)')


def menu(ancho: int = 60) -> None:
    ahora = datetime.now()
    fecha_final = datetime(year=2026, month=2, day=28)
    duracion = fecha_final - ahora
    dias = duracion.days
    opcion: int = -1
    if dias >= 0:
        while opcion != 0:
            clear()
            texto = '{:{align}{width}}'.format('MENU PRINCIPAL', align='^', width=ancho)
            cprint(texto, 'blue', 'on_white')
            # opcion = '{:2d} > {:55}'.format(1, 'PROCESAR MEMORIA')
            # print('{:{align}{width}}'.format(opcion, align='^', width=ancho))

            opcion = '{:2d} > {:55}'.format(1, 'DESCARGAR EXPEDIENTES')
            print('{:{align}{width}}'.format(opcion, align='^', width=ancho))

            opcion = '{:2d} > {:55}'.format(2, 'INFORMACIÓN')
            print('{:{align}{width}}'.format(opcion, align='^', width=ancho))

            opcion = '{:2d} > {:55}'.format(3, 'PROCESAR EXPIDIENTE EN MEMORIA RAM')
            print('{:{align}{width}}'.format(opcion, align='^', width=ancho))

            opcion = '{:2d} > {:55}'.format(0, 'SALIR')
            print('{:{align}{width}}'.format(opcion, align='^', width=ancho))
            print(
                '{:{align}{width}}'.format('DESARROLLADO POR MAURICIO ANDRES ZAMORA HERNÁNDEZ', align='^', width=ancho))

            opcion = '{:3} DIAS PARA QUE DEJE FUNCIONAR ESTA VERSION DE PRUEBA'.format(dias)
            texto = '{:{align}{width}}'.format(opcion, align='^', width=ancho)
            cprint(texto, 'yellow', 'on_red')
            opcion = leer_rango_numeros_enteros('Digite la opción del menú:', 0, 4)
            if opcion == 0:
                opcion_menu_salir()
            elif opcion == 3:
                opcion_menu_procesar_memoria()
            # elif opcion == 4:
            #     opcion_menu_pruebas()
            elif opcion == 2:
                opcion_menu_informacion()
            elif opcion == 1:
                opcion_menu_descargar()

    else:
        print('{:{align}{width}}'.format('SE HA VENCIDO LA VERSION DE PRUEBAS', align='^', width=ancho))
        print('{:{align}{width}}'.format('POR FAVOR CONSULTAR POR LA VERSIÓN ACTUAL', align='^', width=ancho))
        print('{:{align}{width}}'.format('mauricio.zamora@gmail.com', align='^', width=ancho))



from typing import Dict
from xlsxwriter.format import Format
from xlsxwriter.workbook import Workbook
from xlsxwriter.worksheet import Worksheet

from .curso_carrera import CursoCarrera


def escribir_curso(curso: CursoCarrera, cuaderno: Workbook, hoja: Worksheet, fila: int, columna: int,
                   formatos: Dict[str, Format], relleno_historial: int = 0):
    row: int = fila
    col: int = columna

    formato_celda_centrado = formatos['formato_celda_centrado']
    formato_celda_wrap = formatos['formato_celda_wrap']
    formato_celda_wrap_verde = formatos['formato_celda_wrap_verde']
    formato_celda_wrap_rojo = formatos['formato_celda_wrap_rojo']
    formato_celda_wrap_amarillo = formatos['formato_celda_wrap_amarillo']
    formato_celda_centrado_verde = formatos['formato_celda_centrado_verde']
    # formato_celda_centrado_rojo = formatos['formato_celda_centrado_rojo']
    formato_celda_centrado_amarillo = formatos['formato_celda_centrado_amarillo']
    # Bloque 1
    hoja.write(row, col, curso.get_sigla(), formato_celda_centrado)
    row += 1
    hoja.write(row, col, curso.get_creditos(), formato_celda_centrado)
    row += 1
    try:
        nota: float = float(curso.get_nota())
        hoja.merge_range(row, col, row + 1, col, nota, formato_celda_centrado)
    except (ValueError, TypeError):
        hoja.merge_range(row, col, row + 1, col, curso.get_nota(), formato_celda_centrado)

    # Bloque 2
    row = fila
    col += 1

    if curso.get_estado() in ['APROBADO', 'EQUIVALENTE', 'CONVALIDADO']:
        hoja.merge_range(row, col, row + 2, col + 4, curso.get_nombre(), formato_celda_wrap_verde)
    elif curso.get_estado() in ['MATRICULADO']:
        hoja.merge_range(row, col, row + 2, col + 4, curso.get_nombre(), formato_celda_wrap_amarillo)
    elif curso.get_estado() in ['RETIRO DE MA']:
        hoja.merge_range(row, col, row + 2, col + 4, curso.get_nombre(), formato_celda_wrap_rojo)
    elif curso.get_estado() in ['REPROBADO']:
        hoja.merge_range(row, col, row + 2, col + 4, curso.get_nombre(), formato_celda_wrap_rojo)
    else:
        hoja.merge_range(row, col, row + 2, col + 4, curso.get_nombre(), formato_celda_wrap)
    row += 3
    hoja.merge_range(row, col, row, col + 4, curso.get_estado(), formato_celda_wrap)

    # Bloque 3 y 4
    temp_row = fila
    temp_col = col + 5
    for r in range(temp_row, temp_row + 4):
        hoja.write_blank(r, temp_col, None, formato_celda_centrado)
        hoja.write_blank(r, temp_col + 1, None, formato_celda_centrado)

    req_correq: Dict[str, int] = {}
    req_correq.update(curso.get_requisitos())
    req_correq.update(curso.get_correquisitos())
    siglas_rc = list(req_correq.keys())
    siglas_rc.sort()
    if len(siglas_rc) > 4:
        bloque_3 = siglas_rc[:4]
        bloque_4 = siglas_rc[4:]
    else:
        bloque_3 = siglas_rc
        bloque_4 = []

    # Bloque 3
    for r, rcq in enumerate(bloque_3, temp_row):
        if rcq in curso.get_requisitos():
            if curso.get_requisitos()[rcq] == 4:
                f = formato_celda_centrado_verde
            elif curso.get_requisitos()[rcq] == 0:
                f = formato_celda_centrado
            elif curso.get_requisitos()[rcq] == 1:
                f = formato_celda_centrado_amarillo
            else:
                f = formato_celda_centrado
        else:
            if curso.get_correquisitos()[rcq] == 4:
                f = formato_celda_centrado_verde
            elif curso.get_correquisitos()[rcq] == 0:
                f = formato_celda_centrado
            elif curso.get_correquisitos()[rcq] == 1:
                f = formato_celda_centrado_amarillo
            else:
                f = formato_celda_centrado
        hoja.write(r, temp_col, rcq, f)

    # Bloque 4
    for r, rcq in enumerate(bloque_4, temp_row):
        if rcq in curso.get_requisitos():
            # print('req/cor', rcq, curso.py.get_requisitos()[rcq])
            if curso.get_requisitos()[rcq] == 4:
                f = formato_celda_centrado_verde
            elif curso.get_requisitos()[rcq] == 0:
                f = formato_celda_centrado
            elif curso.get_requisitos()[rcq] == 1:
                f = formato_celda_centrado_amarillo
            else:
                f = formato_celda_centrado
        else:
            if curso.get_correquisitos()[rcq] == 4:
                f = formato_celda_centrado_verde
            elif curso.get_correquisitos()[rcq] == 0:
                f = formato_celda_centrado
            elif curso.get_correquisitos()[rcq] == 1:
                f = formato_celda_centrado_amarillo
            else:
                f = formato_celda_centrado
        hoja.write(r, temp_col + 1, rcq, f)

    # Bloque 5
    row += 1
    col -= 1
    for r, h in enumerate(curso.get_historial(), row):
        row += 1
        hoja.write(r, col, h.get_sigla(), formato_celda_centrado)
        hoja.merge_range(r, col + 1, r, col + 5, h.get_nombre(), formato_celda_wrap)
        hoja.write(r, col + 6, '{:1}-{:4}'.format(h.get_periodo(), h.get_anno()), formato_celda_centrado)
        try:
            nota = float(h.get_nota())
            hoja.write_number(r, col + 7, nota, formato_celda_centrado)
        except (ValueError, TypeError):
            hoja.write(r, col + 7, h.get_nota(), formato_celda_centrado)
    lineas_blancas = relleno_historial - len(curso.get_historial())
    for lb in range(lineas_blancas):
        hoja.merge_range(row, col, row, col + 7, '', formato_celda_wrap)
        row += 1



import re
from typing import Optional, Tuple
from .funciones_io import *
from .funciones_imprimir import imprimir_cursos_solicitados


def identificar_tipo(texto: str) -> Optional[str]:
    re_expediente = re.compile('(Expediente académico)')
    expediente = re_expediente.search(texto)
    re_prematricula = re.compile('(Cursos solicitados en prematrícula)')
    prematricula = re_prematricula.search(texto)
    if prematricula:
        return 'pre'
    elif expediente:
        return 'exp'
    else:
        return None


def procesar_cursos_solicitados(texto: str) -> Tuple[str, str]:
    re_carne = re.compile(
        r"Carné:\s+([A-Z]?\d{5})\s*\r\n")
    se_carne = re_carne.search(texto)
    carne = se_carne.group(1)

    re_nombre = re.compile(
        r"Nombre:\s+([\w\s]+)\s*\r\n")
    se_nombre = re_nombre.search(texto)
    nombre = se_nombre.group(1)

    re_comentario_estudiante = re.compile(
        r"Comentario del estudiante\r\n(.*)\r\n", re.IGNORECASE)
    se_comentario_estudiante = re_comentario_estudiante.search(texto)
    comentario_estudiante = se_comentario_estudiante.group(1)
    if comentario_estudiante.startswith('Cantidad de créditos solicitados'):
        comentario_estudiante = ''

    # print('Comentario estudiante:', comentario_estudiante)

    re_comentario_profesor = re.compile(
        r"Comentario hacia el Estudiante\s*\r\n(.*)\r\n", re.IGNORECASE)
    se_comentario_profesor = re_comentario_profesor.search(texto)
    comentario_profesor = se_comentario_profesor.group(1)
    if comentario_profesor.startswith('* Cursos con Declaración Jurada'):
        comentario_profesor = ''
    # print('Comentario profesor:', comentario_profesor)

    # print(carne, nombre)

    re_solicitudes = re.compile(
        r"(\*?)[ ]?([A-Z]{2}\d{4}|[A-Z]{2}-[A-Z]|[A-Z]{2}-[I]{1,3})\s*([\.:\dA-Z\(\) ÁÉÍÓÚÑ]+)\s+(\d{1,2}).*\r\n(.*)\r\n")

    solicitudes = []

    for match in re_solicitudes.finditer(texto):
        declaracion = match.group(1)
        sigla = match.group(2)
        curso = match.group(3)
        creditos = int(match.group(4))
        otros = match.group(5).strip()
        solicitudes.append(
            (sigla, curso, creditos, otros, 'SI' if declaracion == '*' else 'NO'))

    solicitudes = sorted(solicitudes, key=lambda x: (x[0]))
    escribir_cursos_solicitados(carne, [
        'SIGLA', 'CURSO', 'CREDITOS', 'AUTORIZACION', 'DEC'], solicitudes)
    escribir_comentarios(carne, comentario_estudiante, comentario_profesor)
    imprimir_cursos_solicitados(solicitudes)

    return carne, nombre


def procesar_expediente(texto: str) -> Tuple[str, str]:
    re_estudiante = re.compile(
        r"Carné:\s+([A-Z]?\d{5})\s+([\w\s]+)\r\n", re.VERBOSE)
    estudiante = re_estudiante.search(texto)
    carne = estudiante.group(1)
    nombre = estudiante.group(2)
    # print(carne, nombre)

    re_historial = re.compile(
        r"([A-Z]{2}\d{4})\s+([\.:\dA-Z\(\) ÁÉÍÓÚÑ]+)\s+(\d{1,2})\s+(\d{1,3})\s+([I]{1,3})\s+(\d{4})\s+([A-Z ]+)\s+(.+)\r\n")

    historial = []

    for match in re_historial.finditer(texto):
        sigla = match.group(1)
        curso = match.group(2)
        creditos = int(match.group(3))
        grupo = int(match.group(4))
        semestre = match.group(5)
        anno = int(match.group(6))
        estado = match.group(7)
        nota = match.group(8)
        historial.append((sigla, curso, creditos, grupo,
                          semestre, anno, estado, nota))

    historial = sorted(historial, key=lambda x: (x[5], x[4], x[0]))

    escribir_informacion_estudiante('{}'.format(carne), carne, nombre)
    escribir_historial(carne, [
        'SIGLA', 'CURSO', 'CREDITOS', 'GRUPO', 'SEM', 'AÑO', 'ESTADO', 'NOTA'], historial)
    # imprimir_historial(historial)
    return carne, nombre



from html.parser import HTMLParser
from typing import List


class StudentParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.reset()
        self.__lista: List[List[str]] = []
        self.__bandera_table: bool = False
        self.__bandera_tr_header: bool = False
        self.__bandera_tr: bool = False
        self.__bandera_lectura: bool = False
        self.__contador: int = 0
        self.__datos: List[str] = []

    def get_lista(self) -> List[List[str]]:
        self.__lista.sort(key=lambda x: (x[5], x[4], x[0]), reverse=True)
        return self.__lista

    def handle_starttag(self, tag, attrs):
        if tag == 'table':
            self.__bandera_table = True

        if tag == 'tr' and not self.__bandera_tr_header:
            self.__bandera_tr_header = True

        if tag == 'tr' and self.__bandera_tr_header:
            self.__bandera_tr = True
            self.__contador = 0

        if tag == 'td' and self.__bandera_tr:
            self.__bandera_lectura = True

    def handle_data(self, data):
        if self.__bandera_lectura:
            if self.__contador == 0:
                self.__datos = []

            t = data.strip()
            if self.__contador == 4:
                t = ' '.join(data.strip().split())
                t = t.split(' ')
                # if t[0] == 'I':
                #     t[0] = '1'
                # elif t[0] == 'II':
                #     t[0] = '2'
                # elif t[0] == 'III':
                #     t[0] = '3'
                self.__datos.extend(t)
            else:
                if self.__contador == 6:
                    try:
                        t = float(t)
                    except (ValueError, TypeError):
                        t = t.strip()

                self.__datos.append(t)

            if self.__contador == 6:
                self.__lista.append(self.__datos)
            self.__contador += 1

    def handle_endtag(self, tag):
        if tag == 'tr' and self.__bandera_tr_header:
            self.__bandera_tr = False

        if tag == 'td' and self.__bandera_tr_header:
            self.__bandera_lectura = False



url_home = 'https://ematricula.ucr.ac.cr/ematricula/admin/showAdminLogin.do'
url_login = 'https://ematricula.ucr.ac.cr/ematricula/admin/loginAdmin.do'
url_listado = 'https://ematricula.ucr.ac.cr/ematricula/admin/profesorExpedienteEstud.do'
url_comentarios = 'https://ematricula.ucr.ac.cr/ematricula/admin/showComentariosProfEstud.do?c={}'
url_notas = 'https://ematricula.ucr.ac.cr/ematricula/admin/nivelAvance.do?c={}'
data = {'user': '', 'password': ''}
DEBUG = True

detalle_cursos = [
    {'sigla': 'EF-D', 'curso.py': 'ACTIVIDAD DEPORTIVA', 'creditos': 0, 'semestre': 1},
    {'sigla': 'EG-CA', 'curso.py': 'CURSO DE ARTE', 'creditos': 2, 'semestre': 1},
    {'sigla': 'EG-I', 'curso.py': 'CURSO INTEGRADO DE HUMANIDADES I',
     'creditos': 6, 'semestre': 1},
    {'sigla': 'MA1001', 'curso.py': 'CÁLCULO I', 'creditos': 3, 'semestre': 1},
    {'sigla': 'QU0100', 'curso.py': 'QUÍMICA GENERAL I',
     'creditos': 3, 'correquisitos': ['QU0101'], 'semestre': 1},
    {'sigla': 'QU0101', 'curso.py': 'LABORATORIO DE QUÍMICA GENERAL I',
     'creditos': 1, 'correquisitos': ['QU0100'], 'semestre': 1},
    {'sigla': 'RP-1', 'curso.py': 'REPERTORIO', 'creditos': 3, 'semestre': 1},
    {'sigla': 'EG-II', 'curso.py': 'CURSO INTEGRADO DE HUMANIDADES II',
     'creditos': 6, 'semestre': 2},
    {'sigla': 'FS0210', 'curso.py': 'FÍSICA GENERAL I', 'creditos': 3,
     'requisitos': ['MA1001'], 'correquisitos': ['FS0211'], 'semestre': 2},
    {'sigla': 'FS0211', 'curso.py': 'LABORATORIO DE FÍSICA GENERAL I', 'creditos': 1,
     'requisitos': ['MA1001'], 'correquisitos': ['FS0210'], 'semestre': 2},
    {'sigla': 'II0201', 'curso.py': 'INTRODUCCIÓN A LA INGENIERÍA INDUSTRIAL',
     'creditos': 2, 'semestre': 2},
    {'sigla': 'MA1002', 'curso.py': 'CÁLCULO II', 'creditos': 4,
     'requisitos': ['MA1001'], 'semestre': 2},
    {'sigla': 'QU0102', 'curso.py': 'QUÍMICA GENERAL II', 'creditos': 3, 'requisitos': [
        'QU0100', 'QU0101'], 'correquisitos': ['QU0103'], 'semestre': 2},
    {'sigla': 'QU0103', 'curso.py': 'LABORATORIO DE QUÍMICA GENERAL II', 'creditos': 1,
     'requisitos': ['QU0100', 'QU0101'], 'correquisitos': ['QU0102'], 'semestre': 2},
    {'sigla': 'CI0202', 'curso.py': 'PRINCIPIOS DE INFORMÁTICA',
     'creditos': 4, 'requisitos': ['MA0205'], 'semestre': 3},
    {'sigla': 'FS0310', 'curso.py': 'FÍSICA GENERAL II', 'creditos': 3, 'requisitos': [
        'FS0210', 'FS0211', 'MA1002'], 'correquisitos': ['FS0311'], 'semestre': 3},
    {'sigla': 'FS0311', 'curso.py': 'LABORATORIO DE FÍSICA GENERAL II', 'creditos': 1,
     'requisitos': ['FS0210', 'FS0211', 'MA1002'], 'correquisitos': ['FS0310'], 'semestre': 3},
    {'sigla': 'II0306', 'curso.py': 'PROBABILIDAD Y ESTADÍSTICA',
     'creditos': 3, 'requisitos': ['MA1002'], 'semestre': 3},
    {'sigla': 'MA1003', 'curso.py': 'CÁLCULO III', 'creditos': 4,
     'requisitos': ['MA1002'], 'correquisitos': ['MA1004'], 'semestre': 3},
    {'sigla': 'MA1004', 'curso.py': 'ÁLGEBRA LINEAL', 'creditos': 3, 'semestre': 3},
    {'sigla': 'FS0410', 'curso.py': 'FÍSICA GENERAL III', 'creditos': 3, 'requisitos': [
        'FS0310', 'FS0311', 'MA1003'], 'correquisitos': ['FS0411'], 'semestre': 4},
    {'sigla': 'FS0411', 'curso.py': 'LABORATORIO DE FÍSICA GENERAL III', 'creditos': 1,
     'requisitos': ['FS0310', 'FS0311', 'MA0450'], 'correquisitos': ['FS0410'], 'semestre': 4},
    {'sigla': 'II0401', 'curso.py': 'INVESTIGACIÓN DE OPERACIONES',
     'creditos': 3, 'requisitos': ['II0306', 'MA1004'], 'semestre': 4},
    {'sigla': 'II0402', 'curso.py': 'INGENIERÍA DE CALIDAD I',
     'creditos': 2, 'requisitos': ['II0306'], 'semestre': 4},
    {'sigla': 'II0501', 'curso.py': 'TECNOLOGÍAS DE INFORMACIÓN',
     'creditos': 2, 'requisitos': ['CI0202'], 'semestre': 4},
    {'sigla': 'IM0202', 'curso.py': 'DIBUJO I', 'creditos': 3,
     'requisitos': ['FS0210', 'MA1001'], 'semestre': 4},
    {'sigla': 'MA1005', 'curso.py': 'ECUACIONES DIFERENCIALES',
     'creditos': 4, 'requisitos': ['MA1002', 'MA1004'], 'semestre': 4},
    {'sigla': 'IE0303', 'curso.py': 'ELECTROTECNIA I', 'creditos': 3,
     'requisitos': ['FS0310', 'FS0311', 'MA1003'], 'semestre': 5},
    {'sigla': 'II0302', 'curso.py': 'DISEÑO DEL TRABAJO E INGENIERÍA DE FACTORES HUMANOS',
     'creditos': 3, 'requisitos': ['II0306'], 'semestre': 5},
    {'sigla': 'II0502', 'curso.py': 'INGENIERÍA DE CALIDAD II',
     'creditos': 4, 'requisitos': ['II0402'], 'semestre': 5},
    {'sigla': 'II0503', 'curso.py': 'SIMULACIÓN', 'creditos': 3,
     'requisitos': ['CI0202', 'II0401'], 'semestre': 5},
    {'sigla': 'II0504', 'curso.py': 'ADMINISTRACIÓN FINANCIERA Y CONTABLE I',
     'creditos': 2, 'requisitos': ['II0201'], 'semestre': 5},
    {'sigla': 'IM0207', 'curso.py': 'MECÁNICA I', 'creditos': 3, 'requisitos': [
        'FS0210', 'FS0211', 'IM0202', 'MA1002'], 'semestre': 5},
    {'sigla': 'II0601', 'curso.py': 'GESTIÓN DE CALIDAD',
     'creditos': 4, 'requisitos': ['II0502'], 'semestre': 6},
    {'sigla': 'II0603', 'curso.py': 'SISTEMAS AUTOMATIZADOS DE MANUFACTURA', 'creditos': 3,
     'requisitos': ['FS0410', 'FS0411', 'IE0303', 'II0302', 'II0503', 'II0504'], 'semestre': 6},
    {'sigla': 'II0604', 'curso.py': 'ADMINISTRACIÓN FINANCIERA Y CONTABLE II',
     'creditos': 2, 'requisitos': ['II0504'], 'semestre': 6},
    {'sigla': 'II0605', 'curso.py': 'LOGÍSTICA DE LA CADENA DEL VALOR I',
     'creditos': 3, 'requisitos': ['II0401'], 'semestre': 6},
    {'sigla': 'II0606', 'curso.py': 'TERMOFLUIDOS', 'creditos': 3, 'requisitos': [
        'FS0310', 'FS0311', 'IE0303', 'IM0207', 'QU0102', 'QU0103'], 'semestre': 6},
    {'sigla': 'II0701', 'curso.py': 'DISEÑO DE SISTEMAS DE INFORMACIÓN',
     'creditos': 3, 'requisitos': ['II0501'], 'semestre': 6},
    {'sigla': 'II0602', 'curso.py': 'DISEÑO DE EXPERIMENTOS',
     'creditos': 3, 'requisitos': ['II0601'], 'semestre': 7},
    {'sigla': 'II0702', 'curso.py': 'COMPORTAMIENTO ORGANIZACIONAL',
     'creditos': 2, 'requisitos': ['II0302'], 'semestre': 7},
    {'sigla': 'II0703', 'curso.py': 'INGENIERÍA DE OPERACIONES', 'creditos': 4,
     'requisitos': ['II0603', 'II0604', 'II0605', 'II0701'], 'semestre': 7},
    {'sigla': 'II0704', 'curso.py': 'INGENIERÍA ECONÓMICA Y FINANCIERA', 'creditos': 3,
     'requisitos': ['II0604'], 'correquisitos': ['II0703'], 'semestre': 7},
    {'sigla': 'II0705', 'curso.py': 'LOGÍSTICA DE LA CADENA DEL VALOR II',
     'creditos': 4, 'requisitos': ['II0605'], 'semestre': 7},
    {'sigla': 'SR-I', 'curso.py': 'SEMINARIO DE REALIDAD NACIONAL I',
     'creditos': 2, 'semestre': 7},
    {'sigla': 'II0802', 'curso.py': 'INGENIERÍA DE PROCESOS DE NEGOCIO',
     'creditos': 4, 'requisitos': ['II0702'], 'semestre': 8},
    {'sigla': 'II0803', 'curso.py': 'DISEÑO DE PRODUCTO', 'creditos': 3,
     'requisitos': ['II0704'], 'correquisitos': ['II0804'], 'semestre': 8},
    {'sigla': 'II0804', 'curso.py': 'GESTIÓN DE PROYECTOS',
     'creditos': 3, 'requisitos': ['II0704'], 'semestre': 8},
    {'sigla': 'II0805', 'curso.py': 'DISTRIBUCIÓN Y LOCALIZACIÓN DE INSTALACIONES',
     'creditos': 4, 'requisitos': ['II0703', 'II0705'], 'semestre': 8},
    {'sigla': 'II0806', 'curso.py': 'METROLOGÍA Y NORMALIZACIÓN',
     'creditos': 3, 'requisitos': ['II0602'], 'semestre': 8},
    {'sigla': 'SR-II', 'curso.py': 'SEMINARIO DE REALIDAD NACIONAL II',
     'creditos': 2, 'semestre': 8},
    {'sigla': 'II0801', 'curso.py': 'INGENIERÍA DE SERVICIOS', 'creditos': 3,
     'requisitos': ['II0601', 'II0703', 'II0705'], 'semestre': 9},
    {'sigla': 'II0902', 'curso.py': 'PROYECTO INDUSTRIAL', 'creditos': 3, 'semestre': 9},
    {'sigla': 'II0904', 'curso.py': 'INGENIERÍA AMBIENTAL', 'creditos': 3,
     'requisitos': ['II0601', 'II0804'], 'semestre': 9},
    {'sigla': 'II0905', 'curso.py': 'INGENIERÍA DE MANUFACTURA', 'creditos': 3,
     'requisitos': ['II0703', 'II0803', 'II0805'], 'semestre': 9},
    {'sigla': 'II0906', 'curso.py': 'GESTIÓN DE MANTENIMIENTO', 'creditos': 3,
     'requisitos': ['II0601', 'II0603', 'II0606'], 'semestre': 9},
    {'sigla': 'OPT408a', 'curso.py': 'BLOQUE OPTATIVO 408', 'creditos': 3, 'semestre': 9},
    {'sigla': 'II1001', 'curso.py': 'RESPONSABILIDAD SOCIAL',
     'creditos': 2, 'requisitos': ['II0904'], 'semestre': 10},
    {'sigla': 'II9500', 'curso.py': 'INVESTIGACIÓN DIRIGIDA I',
     'creditos': 0, 'requisitos': ['II0902'], 'semestre': 10},
    {'sigla': 'OPT408b', 'curso.py': 'IBLOQUE OPTATIVO 408',
     'creditos': 3, 'semestre': 10},
    {'sigla': 'II9501', 'curso.py': 'INVESTIGACIÓN DIRIGIDA II',
     'creditos': 0, 'semestre': 11},
    {'sigla': 'II9502', 'curso.py': 'INVESTIGACIÓN DIRIGIDA III',
     'creditos': 0, 'semestre': 12}
]



from typing import List, Dict
from .curso_carrera import CursoCarrera


class Semestre:
    def __init__(self, numero: int) -> None:
        self.__numero: int = numero
        self.__cursos: List[CursoCarrera] = []
        self.__cursos_por_sigla: Dict[str, CursoCarrera] = {}
        self.__siglas: List[str] = []

    def obtener_cursos_aprobados(self) -> List[str]:
        return list(map(lambda c: c.get_sigla(),
                        filter(lambda c: c.get_estado() in ['APROBADO', 'EQUIVALENTE', 'CONVALIDADO'],
                               self.get_cursos())))

    def obtener_cursos_matriculados(self) -> List[str]:
        return list(map(lambda c: c.get_sigla(),
                        filter(lambda c: c.get_estado() in ['MATRICULADO'],
                               self.get_cursos())))

    def obtener_cursos_retirados(self) -> List[str]:
        return list(map(lambda c: c.get_sigla(),
                        filter(lambda c: c.get_estado() in ['RETIRO DE MA'],
                               self.get_cursos())))

    def obtener_cursos_reprobados(self) -> List[str]:
        return list(map(lambda c: c.get_sigla(),
                        filter(lambda c: c.get_estado() in ['REPROBADO'],
                               self.get_cursos())))

    def get_numero(self) -> int:
        return self.__numero

    def get_cursos(self) -> List[CursoCarrera]:
        return self.__cursos

    def get_cursos_por_siglas(self) -> Dict[str, CursoCarrera]:
        return self.__cursos_por_sigla

    def get_siglas(self) -> List[str]:
        return self.__siglas

    def agregar_curso(self, curso: CursoCarrera) -> None:
        self.get_cursos().append(curso)
        self.get_siglas().append(curso.get_sigla())
        self.get_cursos_por_siglas()[curso.get_sigla()] = curso
        self.get_siglas().sort()

    def obtener_maximo_historial(self) -> int:
        return max(map(lambda c: len(c.get_historial()), self.get_cursos()))

    def obtener_total_creditos(self) -> int:
        return sum(map(lambda c: c.get_creditos(), self.get_cursos()))

    def obtener_total_creditos_aprobados(self) -> int:
        return sum(
            map(lambda c: c.get_creditos(),
                filter(lambda f: f.get_estado() in ['APROBADO', 'EQUIVALENTE', 'CONVALIDADO'], self.get_cursos())))

    def obtener_maximo_requisitos(self) -> int:
        reqs = list(map(lambda c: len(c.get_requisitos()), filter(lambda f: f.tiene_requisitos(), self.get_cursos())))
        if reqs:
            return max(reqs)
        else:
            return 0

    def obtener_maximo_correquisitos(self) -> int:
        reqs = list(
            map(lambda c: len(c.get_correquisitos()), filter(lambda f: f.tiene_correquisitos(), self.get_cursos())))
        if reqs:
            return max(reqs)
        else:
            return 0

    def esta_completo(self) -> bool:
        return all([c.esta_aprobado() for c in self.get_cursos()])

    def __str__(self) -> str:
        return 'Semestre {:2}'.format(self.get_numero())

    def tiene_curso(self, sigla: str) -> bool:
        return sigla in self.get_siglas()



from typing import Dict
from xlsxwriter.format import Format
from xlsxwriter.workbook import Workbook


def generar_formatos(cuaderno: Workbook) -> Dict[str, Format]:
    formatos: Dict[str, Format] = {}

    formato_celda_wrap = cuaderno.add_format()
    formatos['formato_celda_wrap'] = formato_celda_wrap
    formato_celda_wrap.set_align('center')
    formato_celda_wrap.set_align('vcenter')
    formato_celda_wrap.set_text_wrap()
    formato_celda_wrap.set_border(1)

    formato_celda_wrap_verde = cuaderno.add_format()
    formatos['formato_celda_wrap_verde'] = formato_celda_wrap_verde
    formato_celda_wrap_verde.set_align('center')
    formato_celda_wrap_verde.set_align('vcenter')
    formato_celda_wrap_verde.set_text_wrap()
    formato_celda_wrap_verde.set_bg_color('green')
    formato_celda_wrap_verde.set_border(1)

    formato_celda_wrap_rojo = cuaderno.add_format()
    formatos['formato_celda_wrap_rojo'] = formato_celda_wrap_rojo
    formato_celda_wrap_rojo.set_align('center')
    formato_celda_wrap_rojo.set_align('vcenter')
    formato_celda_wrap_rojo.set_text_wrap()
    formato_celda_wrap_rojo.set_bg_color('red')
    formato_celda_wrap_rojo.set_border(1)

    formato_celda_wrap_amarillo = cuaderno.add_format()
    formatos['formato_celda_wrap_amarillo'] = formato_celda_wrap_amarillo
    formato_celda_wrap_amarillo.set_align('center')
    formato_celda_wrap_amarillo.set_align('vcenter')
    formato_celda_wrap_amarillo.set_text_wrap()
    formato_celda_wrap_amarillo.set_bg_color('yellow')
    formato_celda_wrap_amarillo.set_border(1)

    formato_celda_centrado = cuaderno.add_format()
    formatos['formato_celda_centrado'] = formato_celda_centrado
    formato_celda_centrado.set_align('center')
    formato_celda_centrado.set_align('vcenter')
    formato_celda_centrado.set_border(1)

    formato_celda_centrado_verde = cuaderno.add_format()
    formatos['formato_celda_centrado_verde'] = formato_celda_centrado_verde
    formato_celda_centrado_verde.set_align('center')
    formato_celda_centrado_verde.set_align('vcenter')
    formato_celda_centrado_verde.set_border(1)
    formato_celda_centrado_verde.set_bg_color('green')

    formato_celda_centrado_amarillo = cuaderno.add_format()
    formatos['formato_celda_centrado_amarillo'] = formato_celda_centrado_amarillo
    formato_celda_centrado_amarillo.set_align('center')
    formato_celda_centrado_amarillo.set_align('vcenter')
    formato_celda_centrado_amarillo.set_border(1)
    formato_celda_centrado_amarillo.set_bg_color('yellow')

    formato_celda_centrado_rojo = cuaderno.add_format()
    formatos['formato_celda_centrado_rojo'] = formato_celda_centrado_rojo
    formato_celda_centrado_rojo.set_align('center')
    formato_celda_centrado_rojo.set_align('vcenter')
    formato_celda_centrado_rojo.set_border(1)
    formato_celda_centrado_rojo.set_bg_color('red')

    return formatos



class Curso:
    def __init__(self, sigla: str, nombre: str, creditos: int, nota: str = None, anno: int = None, periodo: int = None,
                 semestre: int = None, estado: str = '', sigla_normalizada: str = None):
        self.__sigla: str = sigla
        self.__sigla_normalizada: str = sigla_normalizada
        self.__nombre: str = nombre
        self.__creditos: int = creditos
        self.__semestre: int = semestre
        self.__estado: str = estado
        self.__nota: str = nota
        self.__anno: int = anno
        self.__periodo: int = periodo

    def get_sigla(self) -> str:
        return self.__sigla

    def set_sigla(self, sigla: str) -> None:
        self.__sigla = sigla

    def get_sigla_normalizada(self) -> str:
        if self.__sigla_normalizada is None:
            return self.get_sigla()
        else:
            return self.__sigla_normalizada

    def set_sigla_normalizada(self, sigla_normalizada: str) -> None:
        self.__sigla_normalizada = sigla_normalizada

    def get_nombre(self) -> str:
        return self.__nombre

    def set_nombre(self, nombre: str) -> None:
        self.__nombre = nombre

    def get_creditos(self) -> int:
        return self.__creditos

    def set_creditos(self, creditos: int) -> None:
        self.__creditos = creditos

    def get_semestre(self) -> int:
        return self.__semestre

    def set_semestre(self, semestre: int) -> None:
        self.__semestre = semestre

    def get_estado(self) -> str:
        return self.__estado

    def set_estado(self, estado: str) -> None:
        self.__estado = estado

    def __str__(self):
        return '{:8} {:8} {:75} {:2d} {:15}'.format(self.get_sigla(), self.get_sigla_normalizada(), self.get_nombre(),
                                                    self.__creditos, self.get_estado())

    def get_nota(self) -> str:
        return self.__nota

    def set_nota(self, nota: str) -> None:
        self.__nota = nota

    def get_anno(self) -> int:
        return self.__anno

    def set_anno(self, anno: int) -> None:
        self.__anno = anno

    def get_periodo(self) -> int:
        return self.__periodo

    def set_periodo(self, periodo: int) -> None:
        self.__periodo = periodo



from typing import List


def imprimir_cursos_solicitados(cursos_solicitados: List) -> None:
    print('{:6} {:70} {:8} {:14} {:^3}'.format(
        'SIGLA', 'CURSO', 'CREDITOS', 'AUTORIZACION', 'DEC'))
    for cs in cursos_solicitados:
        sigla, curso, creditos, autorizacion, dec = cs
        print('{:6} {:70} {:^8d} {:^14} {:^3}'.format(
            sigla, curso, creditos, autorizacion, dec))


def imprimir_historial(historial: List) -> None:
    print('{:6} {:70} {:8} {:5} {:3} {:>4} {:>12} {:>4}'.format(
        'SIGLA', 'CURSO', 'CREDITOS', 'GRUPO', 'SEM', 'anno', 'ESTADO', 'NOTA'))
    for h in historial:
        sigla, curso, creditos, grupo, semestre, anno, estado, nota = h
        print('{:6} {:70} {:^8d} {:5d} {:>3} {:4d} {:12} {:>4}'.format(
            sigla, curso, creditos, grupo, semestre, anno, estado, nota))



from os import system, name


def clear() -> None:
    # for windows
    if name == 'nt':
        _ = system('cls')
    # for mac and linux
    else:
        _ = system('clear')


def leer_numero_entero(etiqueta: str = None) -> int:
    if etiqueta is not None:
        print(etiqueta)
    continuar: bool = True
    numero_entero: int = 0
    while continuar:
        numero = input()
        continuar = False
        try:
            numero_entero = int(numero)
        except (ValueError, TypeError):
            continuar = True
            print('El valor no es entero, por favor reintente: ')

    return numero_entero


def leer_rango_numeros_enteros(etiqueta: str = None, rango_inferior: int = 0, rango_superior: int = 99999999) -> int:
    numero_entero: int = leer_numero_entero(etiqueta)
    continuar: bool = True
    while continuar:
        if rango_inferior <= numero_entero <= rango_superior:
            continuar = False
        else:
            print('Debe digitar un número entre {} y {}'.format(rango_inferior, rango_superior))
            numero_entero: int = leer_numero_entero(etiqueta)

    return numero_entero



import requests
from .config import *
from .student_parser import StudentParser
from .main_listing_parser import MainListingParser
from .funciones_io import escribir_historial, escribir_informacion_estudiante
from .custom_http_adapter import CustomHttpAdapter, get_legacy_session
import re
from termcolor import cprint
import urllib3
import ssl

def procesar_expediente_estudiante(texto):
    student_parser = StudentParser()
    student_parser.feed(texto)
    return student_parser.get_lista()


def descargar_expediente_estudiante(carne, nombre, clave, sess):
    r = sess.get(url_notas.format(clave))
    return r.text


def iniciar_proceso_descarga() -> bool:
    se_puedo_descargar = False
    # s = requests.Session()
    # s.post(url_login, data=data)
    s = get_legacy_session(data, url_login)
    r = s.get(url_listado)
    contenido = r.text
    match = re.findall(r'Listado de estudiantes asignados al profesor', contenido)

    if match:
        se_puedo_descargar = True
        main_listing_parser = MainListingParser()
        main_listing_parser.feed(contenido)
        estudiantes = main_listing_parser.get_lista()
        textos = []
        for estudiante in estudiantes:
            clave = estudiante[0]
            carne = estudiante[1:2][0]
            nombre = estudiante[2:3][0]
            correo = estudiante[-1]
            texto = descargar_expediente_estudiante(carne, nombre, clave, s)
            textos.append(texto)
            lineas = procesar_expediente_estudiante(texto)
            escribir_historial(carne, ['SIGLA', 'CURSO', 'CREDITOS', 'GRUPO', 'SEM', 'AÑO', 'ESTADO', 'NOTA'], lineas)
            escribir_informacion_estudiante(carne, carne, nombre)


    else:
        cprint('NO SE HA PODIDO OBTENER LA INFORMACIÓN, FAVOR VERIFIQUE SUS CREDENCIALES (USUARIO Y CLAVE)', 'white',
               'on_red', attrs=['bold'])

    return se_puedo_descargar



import urllib3
import ssl
import requests


class CustomHttpAdapter(requests.adapters.HTTPAdapter):
    # "Transport adapter" that allows us to use custom ssl_context.

    def __init__(self, ssl_context=None, **kwargs):
        self.ssl_context = ssl_context
        super().__init__(**kwargs)

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = urllib3.poolmanager.PoolManager(
            num_pools=connections, maxsize=maxsize,
            block=block, ssl_context=self.ssl_context)


def get_legacy_session(session_login_data, url):
    ctx = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
    ctx.options |= 0x4  # OP_LEGACY_SERVER_CONNECT
    session = requests.session()
    session.mount('https://', CustomHttpAdapter(ctx))
    # s = requests.Session()
    session.post(url, data=session_login_data)
    return session



from typing import List, Dict, Optional
from .curso import Curso
from .historial import Historial


class CursoCarrera(Curso):
    def __init__(self, sigla: str, nombre: str, creditos: int, nota: str = None, anno: int = None, periodo: int = None,
                 semestre: int = None, estado: str = ''):
        super().__init__(sigla, nombre, creditos, nota, anno, periodo, semestre, estado)
        self.__historial: List[Historial] = []
        self.__requisitos: Dict[str, int] = {}  # 0 Sin datos, 1 Matriculado 2 Reprobado 3 Retirado 4 Aprobado 5 Conv.
        self.__correquisitos: Dict[
            str, int] = {}  # 0 Sin datos, 1 Matriculado 2 Reprobado 3 Retirado 4 Aprobado 5 Conv.

    def get_historial(self) -> List[Historial]:
        return self.__historial

    def get_requisitos(self) -> Dict[str, int]:
        return self.__requisitos

    def tiene_requisitos(self) -> bool:
        return bool(self.get_requisitos())

    def agregar_requisito(self, sigla: str) -> None:
        self.get_requisitos()[sigla] = 0

    def cumple_requisitos(self) -> bool:
        requisitos: List[bool] = [x == 4 for x in self.get_requisitos().values()]
        return not requisitos or all(requisitos)

    def get_correquisitos(self) -> Dict[str, int]:
        return self.__correquisitos

    def tiene_correquisitos(self) -> bool:
        return bool(self.get_correquisitos())

    def agregar_correquisito(self, sigla: str) -> None:
        self.get_correquisitos()[sigla] = 0

    def cumple_correquisitos(self) -> bool:
        return not self.get_correquisitos().values() or all([x == 4 for x in self.get_correquisitos().values()])

    def verificar_requisitos_correquisitos_aprobados(self, siglas_cursos_aprobados: List[str]):
        if siglas_cursos_aprobados:
            for sigla in self.get_requisitos().keys():
                if sigla in siglas_cursos_aprobados:
                    self.get_requisitos()[sigla] = 4
            for sigla in self.get_correquisitos().keys():
                if sigla in siglas_cursos_aprobados:
                    self.get_correquisitos()[sigla] = 4

    def verificar_requisitos_correquisitos_matriculados(self, siglas_cursos_matriculados: List[str]):
        if siglas_cursos_matriculados:
            for sigla in self.get_requisitos().keys():
                if sigla in siglas_cursos_matriculados:
                    self.get_requisitos()[sigla] = 1
            for sigla in self.get_correquisitos().keys():
                if sigla in siglas_cursos_matriculados:
                    self.get_correquisitos()[sigla] = 1

    def verificar_requisitos_correquisitos_reprobados(self, siglas_cursos_reprobados: List[str]):
        if siglas_cursos_reprobados:
            for sigla in self.get_requisitos().keys():
                if sigla in siglas_cursos_reprobados:
                    self.get_requisitos()[sigla] = 2
            for sigla in self.get_correquisitos().keys():
                if sigla in siglas_cursos_reprobados:
                    self.get_correquisitos()[sigla] = 2

    def verificar_requisitos_correquisitos_retirados(self, siglas_cursos_retirados: List[str]):
        if siglas_cursos_retirados:
            for sigla in self.get_requisitos().keys():
                if sigla in siglas_cursos_retirados:
                    self.get_requisitos()[sigla] = 3
            for sigla in self.get_correquisitos().keys():
                if sigla in siglas_cursos_retirados:
                    self.get_correquisitos()[sigla] = 3

    def agregar_historial(self, historial: Historial):
        self.__historial.append(historial)
        self.__historial.sort(key=lambda h: (
            h.get_anno(), h.get_periodo()), reverse=True)

    def get_nota(self) -> str:
        if self.get_historial():
            return self.get_historial()[0].get_nota()
        else:
            return ''

    def get_estado(self) -> str:
        if self.get_historial():
            return self.get_historial()[0].get_estado()
        else:
            return ''

    def esta_aprobado(self) -> bool:
        return self.get_estado() in ['APROBADO', 'EQUIVALENTE', 'CONVALIDADO']

    def get_anno(self) -> Optional[int]:
        if self.get_historial():
            return self.get_historial()[0].get_anno()
        else:
            return None

    def get_periodo(self) -> Optional[int]:
        if self.get_historial():
            return self.get_historial()[0].get_periodo()
        else:
            return None

    def __str__(self):
        periodo = self.get_periodo() if not self.get_periodo() is None else ''
        anno = self.get_anno() if not self.get_anno() is None else ''
        salida: str = '\t' + '{} {:>3} {:4} {:4}'.format(super().__str__(), periodo, anno, self.get_nota())
        salida = salida + '\n'
        if self.get_historial():
            salida = salida + '\t\tHistorial:'
            for h in self.get_historial():
                salida = salida + '\n\t\t' + str(h)
        else:
            salida = salida + '\t\t' + 'Sin Historial'
        return salida



class Historial:
    def __init__(self, sigla: str, sigla_normalizada: str, nombre: str, grupo: int, periodo: int, anno: int,
                 estado: str,
                 nota: str):
        self.__sigla: str = sigla
        self.__sigla_normalizada: str = sigla_normalizada
        self.__nombre: str = nombre
        self.__grupo: int = grupo
        self.__periodo: int = periodo
        self.__anno: int = anno
        self.__estado: str = estado
        self.__nota: str = nota

    def get_sigla(self) -> str:
        return self.__sigla

    def set_sigla(self, sigla: str) -> None:
        self.__sigla = sigla

    def get_sigla_normalizada(self) -> str:
        return self.__sigla_normalizada

    def set_sigla_normalizada(self, sigla_normalizada: str) -> None:
        self.__sigla_normalizada = sigla_normalizada

    def get_nombre(self) -> str:
        return self.__nombre

    def set_nombre(self, nombre: str) -> None:
        self.__nombre = nombre

    def get_grupo(self) -> int:
        return self.__grupo

    def set_grupo(self, grupo: int) -> None:
        self.__grupo = grupo

    def get_periodo(self) -> int:
        return self.__periodo

    def set_periodo(self, periodo: int) -> None:
        self.__periodo = periodo

    def get_anno(self) -> int:
        return self.__anno

    def set_anno(self, anno: int) -> None:
        self.__anno = anno

    def get_estado(self) -> str:
        return self.__estado

    def set_estado(self, estado: int) -> None:
        self.__estado = estado

    def get_nota(self) -> str:
        return self.__nota

    def set_nota(self, nota: str) -> None:
        self.__nota = nota

    def __str__(self):
        if self.get_nota() is None:
            nota = ''
        else:
            nota = self.get_nota()

        return '{:8} {:8} {:75} {:3} {:4} {:4} {:15}'.format(self.get_sigla(), self.get_sigla_normalizada(),
                                                             self.get_nombre(), self.get_periodo(), self.get_anno(),
                                                             nota, self.get_estado())



from typing import Dict, Tuple
from xlsxwriter.format import Format
from xlsxwriter.workbook import Workbook
from xlsxwriter.worksheet import Worksheet

from .expediente import Expediente
from .funciones_semestre import escribir_semestre_listado, escribir_semestre


def escribir_encabezado_expediente(expediente: Expediente, cuaderno: Workbook, hoja: Worksheet, fila: int, columna: int,
                                   formatos: Dict[str, Format]) -> Tuple[int, int]:
    row = fila
    col = columna
    formato_celda_wrap = formatos['formato_celda_wrap']
    hoja.merge_range(row, col, row, col + 1, 'CARNET:', formato_celda_wrap)
    hoja.merge_range(row, col + 2, row, col + 7, expediente.get_carne(), formato_celda_wrap)
    row += 1
    hoja.merge_range(row, col, row, col + 1, 'NOMBRE:', formato_celda_wrap)
    hoja.merge_range(row, col + 2, row, col + 7, expediente.get_nombre(), formato_celda_wrap)
    row += 2
    return row, col


def escribir_expediente_listado(expediente: Expediente, cuaderno: Workbook, hoja: Worksheet, fila: int, columna: int,
                                formatos: Dict[str, Format]):
    # print('Escribir expediente')
    row = fila
    col = columna

    lista_semestres = expediente.obtener_semestres().keys()
    for id_semestre in lista_semestres:
        row, col = escribir_semestre_listado(expediente.obtener_semestres()[id_semestre], cuaderno, hoja, row, col,
                                             formatos)
        # print('rc-', row, col)
        # print(expediente.obtener_semestres()[id_semestre].obtener_cursos_aprobados())


def escribir_expediente(expediente: Expediente, cuaderno: Workbook, hoja: Worksheet, fila: int, columna: int,
                        formatos: Dict[str, Format]):
    # print('Escribir expediente')
    row = fila
    col = columna

    lista_semestres = expediente.obtener_semestres().keys()
    for id_semestre in lista_semestres:
        row, col = escribir_semestre(expediente.obtener_semestres()[id_semestre], cuaderno, hoja, row, col, formatos)
        # print('rc-', row, col)
        # print(expediente.obtener_semestres()[id_semestre].obtener_cursos_aprobados())



import base64

from html.parser import HTMLParser
from typing import List


class MainListingParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.reset()
        self.__lista: List[List[str]] = []
        self.__estudiante_detectado: bool = False
        self.__contador: int = 0
        self.__datos: List[str] = []

    def get_lista(self) -> List[List[str]]:
        return self.__lista

    def handle_starttag(self, tag, attrs):
        if tag == 'input':
            dic = dict(attrs)
            if 'name' in dic and dic['name'] == 'radio':
                clave = '!!'.join(dic['value'].split(','))
                clave = base64.b64encode(clave.encode("utf-8")).decode("utf-8")
                self.__estudiante_detectado = True
                self.__datos = []
                self.__datos.append(clave)
                self.__lista.append(self.__datos)

        if tag == 'td' and self.__estudiante_detectado:
            self.__contador += 1
            if self.__contador == 5:
                self.__contador = 0
                self.__estudiante_detectado = False
                # self.__lista.append(self.__datos)

    def handle_data(self, data):
        if self.__estudiante_detectado:
            if data.strip() != '':
                self.__datos.append(data.strip())



import os
import xlsxwriter

from datetime import timedelta
from typing import List, Dict, Tuple
from termcolor import cprint
from xlsxwriter.format import Format
from xlsxwriter.workbook import Workbook
from xlsxwriter.worksheet import Worksheet

from .config import *
from .semestre import Semestre
from .curso_carrera import CursoCarrera
from .expediente import Expediente
from .funciones_io import leer_historial
from .funciones_xlsxwriter import generar_formatos
from .funciones_expediente import escribir_encabezado_expediente, escribir_expediente


def cargar_cursos_carrera(listado_cursos: List) -> Tuple[Dict[str, CursoCarrera], Dict[int, Semestre], Expediente]:
    sigla_cursos: Dict[str, CursoCarrera] = {}
    semestre_cursos: Dict[int, Semestre] = {}
    expediente = Expediente()
    for detalle_curso in listado_cursos:
        sigla = detalle_curso['sigla']
        nombre = detalle_curso['curso.py']
        semestre = detalle_curso['semestre']
        creditos = detalle_curso['creditos']
        curso = CursoCarrera(sigla, nombre, creditos, semestre=semestre)

        if 'requisitos' in detalle_curso:
            requisitos = detalle_curso['requisitos']
            for requisito in requisitos:
                curso.agregar_requisito(requisito)

        if 'correquisitos' in detalle_curso:
            correquisitos = detalle_curso['correquisitos']
            for correquisito in correquisitos:
                curso.agregar_correquisito(correquisito)

        sigla_cursos[sigla] = curso
        if semestre not in semestre_cursos:
            semestre_cursos[semestre] = Semestre(semestre)
        semestre_cursos[semestre].agregar_curso(curso)

    for sem in semestre_cursos.values():
        expediente.agregar_semestre(sem)

    return sigla_cursos, semestre_cursos, expediente


def procesar_archivos_expedientes() -> None:
    lista_archivos = os.listdir('./expediente')
    impar: bool = True
    tiempo_total = timedelta(seconds=0)
    texto = '{:9} {:69} {:>9} {:>10}'.format('CARNE', 'NOMBRE', 'LINEAS', 'TIEMPO')
    cprint(texto, 'magenta', 'on_yellow')
    for archivo in lista_archivos:
        if archivo.endswith('.edf'):
            with open(os.path.join('./expediente', '{}'.format(archivo))) as f:
                lineas = f.readlines()
                lineas = [linea if not linea.endswith('\n') else linea[:-1] for linea in lineas]
                carne = lineas[0]
                nombre = lineas[1]

                # with open(os.path.join('./expediente', '{}.sdf'.format(carne))) as f:

                sigla_cursos, semestre_cursos, expediente = cargar_cursos_carrera(detalle_cursos)
                expediente.set_carne(carne)
                expediente.set_nombre(nombre)

                lineas = leer_historial(carne)
                for linea in lineas:
                    expediente.agregar_curso(linea)

                # MZ
                # print('MZ', expediente.obtener_semestres_completos())
                tiempo = timedelta(minutes=1) + len(lineas) * timedelta(seconds=20)
                tiempo_total += tiempo
                impar = not impar

                texto = '{:9} {:69} {:9} {:>10}'.format(carne, nombre, len(lineas), str(tiempo))
                if impar:
                    cprint(texto, 'blue', 'on_white')
                else:
                    cprint(texto, 'blue', 'on_cyan')

                existe_directorio: bool = os.path.isdir('./salida')
                if not existe_directorio:
                    os.mkdir('./salida')
                workbook: Workbook = xlsxwriter.Workbook(
                    os.path.join('./salida', '{}-{}.xlsx'.format(carne, nombre.upper())))

                worksheet: Worksheet = workbook.add_worksheet('malla')
                worksheet.hide_gridlines(2)

                formatos: Dict[str, Format] = generar_formatos(workbook)
                r, c = escribir_encabezado_expediente(expediente, workbook, worksheet, 0, 0, formatos)
                expediente.procesar_requisitos_correquisitos()
                escribir_expediente(expediente, workbook, worksheet, r, c, formatos)
                workbook.close()

    texto = 'TIEMPO TOTAL ESTIMADO QUE USTED SE AHORRO SOLO EN ESTA ETAPA: {}'.format(tiempo_total)
    texto = '{:^100}'.format(texto)
    cprint(texto, 'cyan', 'on_red', attrs=['bold', 'blink'])



import os
import csv
from typing import List


def escribir_cursos_solicitados(archivo: str, encabezado: List, cursos_solicitados: List) -> None:
    existe_directorio: bool = os.path.isdir('./solicitudes')
    if not existe_directorio:
        os.mkdir('./solicitudes')
    with open(os.path.join('./solicitudes', '{}.sdf'.format(archivo)), 'w', newline='') as file:
        writer = csv.writer(file, dialect='excel', delimiter='\t')
        writer.writerow(encabezado)
        for h in cursos_solicitados:
            writer.writerow(h)


def escribir_comentarios(archivo: str, estudiante: str, rev: str) -> None:
    existe_directorio: bool = os.path.isdir('./solicitudes')
    if not existe_directorio:
        os.mkdir('./solicitudes')
    f = open(os.path.join('./solicitudes', '{}.edf'.format(archivo)), "w")
    f.write('estudiante:')
    f.write(estudiante)
    f.write('\n')
    f.write('rev:')
    f.write(rev)
    f.close()


def escribir_historial(archivo: str, encabezado: List, historial: List) -> None:
    existe_directorio: bool = os.path.isdir('./expediente')
    if not existe_directorio:
        os.mkdir('./expediente')
    with open(os.path.join('./expediente', '{}.sdf'.format(archivo)), 'w', newline='') as file:
        writer = csv.writer(file, dialect='excel', delimiter='\t')
        writer.writerow(encabezado)
        for h in historial:
            writer.writerow(h)


def leer_historial(archivo: str) -> List:
    salida: List = []
    with open(os.path.join('./expediente', '{}.sdf'.format(archivo)), 'r', newline='') as file:
        reader = csv.DictReader(file, delimiter='\t', dialect='excel')
        # reader = csv.reader(file, delimiter='\t', dialect='excel')
        for linea in reader:
            salida.append(dict(linea))
    return salida


def escribir_informacion_estudiante(archivo: str, carne: str, nombre: str) -> None:
    existe_directorio: bool = os.path.isdir('./expediente')
    if not existe_directorio:
        os.mkdir('./expediente')
    f = open(os.path.join('./expediente', '{}.edf'.format(archivo)), "w")
    f.write(carne)
    f.write('\n')
    f.write(nombre)
    f.close()



from typing import List, Dict, Tuple
from .semestre import Semestre
from .curso_carrera import CursoCarrera
from .historial import Historial


class Expediente:
    def __init__(self) -> None:
        self.__semestres: Dict[int, Semestre] = {}
        self.__siglas: List[str] = []
        self.__optativos: List = []
        self.__otros_cursos: List = []
        self.__carne: str = ''
        self.__nombre: str = ''

    def get_carne(self) -> str:
        return self.__carne

    def set_carne(self, carne: str):
        if carne is not None:
            self.__carne = carne

    def get_nombre(self) -> str:
        return self.__nombre

    def set_nombre(self, nombre: str):
        if nombre is not None:
            self.__nombre = nombre

    def get_siglas(self) -> List[str]:
        return self.__siglas

    def get_optativos(self) -> List[str]:
        return self.__optativos

    def get_otros_cursos(self) -> List[str]:
        return self.__otros_cursos

    def obtener_semestres(self) -> Dict[int, Semestre]:
        return self.__semestres

    def agregar_semestre(self, semestre: Semestre) -> None:
        self.obtener_semestres()[semestre.get_numero()] = semestre
        self.get_siglas().extend(semestre.get_siglas())

    def obtener_semestres_completos(self) -> List[Tuple[int, bool]]:
        salida: List[Tuple[int, bool]] = []
        for i, c in self.obtener_semestres().items():
            salida.append((i, c.esta_completo()))
        return salida

    def agregar_curso(self, dicc: Dict[str, str]) -> None:
        sigla = dicc['SIGLA']
        sigla_normalizada = self.convertir_sigla_normalizada(sigla)
        nombre = dicc['CURSO']
        try:
            grupo = int(dicc['GRUPO'])
        except (ValueError, TypeError):
            grupo = int(0)

        if dicc['SEM'] == 'I':
            periodo = 1
        elif dicc['SEM'] == 'II':
            periodo = 2
        else:
            periodo = 3
        try:
            anno = int(dicc['AÑO'])
        except (ValueError, TypeError):
            anno = int(0)

        estado = dicc['ESTADO']
        nota = dicc['NOTA']
        try:
            nota = str(float(nota))
        except (ValueError, TypeError):
            if nota is None:
                nota = ''
        # print(dicc)
        encontro: bool = False
        for semestre in self.obtener_semestres().values():
            if semestre.tiene_curso(sigla_normalizada):
                encontro = True
                curso: CursoCarrera = semestre.get_cursos_por_siglas()[
                    sigla_normalizada]
                historial = Historial(
                    sigla, sigla_normalizada, nombre, grupo, periodo, anno, estado, nota)
                curso.agregar_historial(historial)
                # print(sigla, sigla_normalizada, ' en ', semestre)

        if not encontro:
            if sigla.startswith('II'):
                self.get_optativos().append(dicc)
            else:
                self.get_otros_cursos().append(dicc)
            # print(sigla, sigla_normalizada, ' no se encontro')

    def obtener_cursos_aprobados(self) -> List[str]:
        lista: List[str] = []
        for semestre in self.obtener_semestres().values():
            lista.extend(semestre.obtener_cursos_aprobados())

        lista.sort()
        return lista

    def obtener_cursos_matriculados(self) -> List[str]:
        lista: List[str] = []
        for semestre in self.obtener_semestres().values():
            lista.extend(semestre.obtener_cursos_matriculados())

        lista.sort()
        return lista

    def obtener_cursos_retirados(self) -> List[str]:
        lista: List[str] = []
        for semestre in self.obtener_semestres().values():
            lista.extend(semestre.obtener_cursos_retirados())

        lista.sort()
        return lista

    def obtener_cursos_reprobados(self) -> List[str]:
        lista: List[str] = []
        for semestre in self.obtener_semestres().values():
            lista.extend(semestre.obtener_cursos_reprobados())

        lista.sort()
        return lista

    def procesar_requisitos_correquisitos(self):
        # print('procesar_requisitos_correquisitos(self):')
        lista = self.obtener_cursos_aprobados()
        for semestre in self.obtener_semestres().values():
            for c in semestre.get_cursos():
                c.verificar_requisitos_correquisitos_aprobados(lista)

        lista = self.obtener_cursos_matriculados()
        for semestre in self.obtener_semestres().values():
            for c in semestre.get_cursos():
                c.verificar_requisitos_correquisitos_matriculados(lista)

        lista = self.obtener_cursos_reprobados()
        for semestre in self.obtener_semestres().values():
            for c in semestre.get_cursos():
                c.verificar_requisitos_correquisitos_reprobados(lista)

        lista = self.obtener_cursos_retirados()
        for semestre in self.obtener_semestres().values():
            for c in semestre.get_cursos():
                c.verificar_requisitos_correquisitos_retirados(lista)

    @staticmethod
    def convertir_sigla_normalizada(sigla: str) -> str:
        sigla_normalizada: str = sigla

        if sigla.startswith('EF'):
            sigla_normalizada = 'EF-D'
        elif sigla.startswith('RP'):
            sigla_normalizada = 'RP-1'
        elif sigla.startswith('EG03'):
            sigla_normalizada = 'EG-CA'
        elif sigla.startswith('EG0124'):
            sigla_normalizada = 'EG-I'
        elif sigla.startswith('EG0126'):
            sigla_normalizada = 'EG-I'
        elif sigla.startswith('EG0125'):
            sigla_normalizada = 'EG-II'
        elif sigla.startswith('EG0127'):
            sigla_normalizada = 'EG-II'
        elif sigla.startswith('SR0001'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0002'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0003'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0004'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0005'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0006'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0007'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0008'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0010'):
            sigla_normalizada = 'SR-I'
        elif sigla.startswith('SR0011'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0022'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0033'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0044'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0055'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0066'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0077'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0088'):
            sigla_normalizada = 'SR-II'
        elif sigla.startswith('SR0110'):
            sigla_normalizada = 'SR-II'
        return sigla_normalizada
